# -*- coding: utf-8 -*-
"""
conversion tags for the mwtab format
"""

import copy


ms_directives = \
{
  "ANALYSIS": {
    "ANALYSIS_TYPE": {
      "id": "ANALYSIS_TYPE",
      "override": "MS",
      "value_type": "str"
    }
  },
  "CHROMATOGRAPHY": {
    "CHROMATOGRAPHY_SUMMARY": {
      "fields": [
        "chromatography_description"
      ],
      "id": "CHROMATOGRAPHY_SUMMARY",
      "required": "False",
      "table": "protocol",
      "test": "machine_type=MS",
      "value_type": "str"
    },
    "CHROMATOGRAPHY_TYPE": {
      "fields": [
        "chromatography_type"
      ],
      "id": "CHROMATOGRAPHY_TYPE",
      "required": "True",
      "table": "protocol",
      "test": "machine_type=MS",
      "value_type": "str"
    },
    "COLUMN_NAME": {
      "fields": [
        "column_name"
      ],
      "id": "COLUMN_NAME",
      "required": "True",
      "table": "protocol",
      "test": "machine_type=MS",
      "value_type": "str"
    },
    "INSTRUMENT_NAME": {
      "fields": [
        "chromatography_instrument_name"
      ],
      "id": "INSTRUMENT_NAME",
      "required": "True",
      "table": "protocol",
      "test": "machine_type=MS",
      "value_type": "str"
    }
  },
  "COLLECTION": {
    "COLLECTION_PROTOCOL_FILENAME": {
      "fields": [
        "filename"
      ],
      "id": "COLLECTION_PROTOCOL_FILENAME",
      "required": "False",
      "table": "protocol",
      "test": "type=collection",
      "value_type": "str"
    },
    "COLLECTION_PROTOCOL_ID": {
      "fields": [
        "id"
      ],
      "id": "COLLECTION_PROTOCOL_ID",
      "required": "True",
      "table": "protocol",
      "test": "type=collection",
      "value_type": "str"
    },
    "COLLECTION_SUMMARY": {
      "fields": [
        "description"
      ],
      "id": "COLLECTION_SUMMARY",
      "required": "True",
      "table": "protocol",
      "test": "type=collection",
      "value_type": "str"
    },
    "SAMPLE_TYPE": {
      "fields": [
        "sample_type"
      ],
      "id": "SAMPLE_TYPE",
      "required": "True",
      "table": "protocol",
      "test": "type=collection",
      "value_type": "str"
    }
  },
  "METABOLOMICS WORKBENCH": {
    "ANALYSIS_ID": {
      "id": "ANALYSIS_ID",
      "override": "AN000000",
      "value_type": "str"
    },
    "CREATED_ON": {
      "code": "str(datetime.datetime.now().date())",
      "id": "CREATED_ON",
      "value_type": "str"
    },
    "STUDY_ID": {
      "id": "STUDY_ID",
      "override": "ST000000",
      "value_type": "str"
    },
    "VERSION": {
      "id": "VERSION",
      "override": "1",
      "value_type": "str"
    }
  },
  "MS": {
    "INSTRUMENT_NAME": {
      "fields": [
        "instrument"
      ],
      "id": "INSTRUMENT_NAME",
      "required": "True",
      "table": "protocol",
      "test": "machine_type=MS",
      "value_type": "str"
    },
    "INSTRUMENT_TYPE": {
      "fields": [
        "instrument_type"
      ],
      "id": "INSTRUMENT_TYPE",
      "required": "True",
      "table": "protocol",
      "test": "machine_type=MS",
      "value_type": "str"
    },
    "ION_MODE": {
      "fields": [
        "ion_mode"
      ],
      "id": "ION_MODE",
      "required": "True",
      "table": "protocol",
      "test": "machine_type=MS",
      "value_type": "str"
    },
    "MS_COMMENTS": {
      "fields": [
        "description"
      ],
      "id": "MS_COMMENTS",
      "required": "False",
      "table": "protocol",
      "test": "machine_type=MS",
      "value_type": "str"
    },
    "MS_TYPE": {
      "fields": [
        "ionization"
      ],
      "id": "MS_TYPE",
      "required": "True",
      "table": "protocol",
      "test": "machine_type=MS",
      "value_type": "str"
    }
  },
  "MS_METABOLITE_DATA": {
    "Data": {
      "required": "True",
      "collate": "assignment",
      "headers": [
        "\"Metabolite\"=assignment",
        "entity.id=intensity"
      ],
      "id": "Data",
      "sort_by": [
        "assignment"
      ],
      "sort_order": "ascending",
      "table": "measurement",
      "value_type": "matrix",
      "values_to_str": "True"
    },
    "Extended": {
      "required": "False",
      "exclusion_headers": [
        "id",
        "intensity",
        "intensity%type",
        "intensity%units",
        "assignment",
        "assignment%method",
        "entity.id",
        "protocol.id",
        "formula",
        "compound",
        "isotopologue",
        "isotopologue%type"
      ],
      "fields_to_headers": "True",
      "headers": [
        "\"Metabolite\"=assignment",
        "\"sample_id\"=entity.id"
      ],
      "id": "Extended",
      "sort_by": [
        "assignment"
      ],
      "sort_order": "ascending",
      "table": "measurement",
      "value_type": "matrix",
      "values_to_str": "True"
    },
    "Metabolites": {
      "required": "True",
      "collate": "assignment",
      "headers": [
        "\"Metabolite\"=assignment"
      ],
      "optional_headers": [
          "assignment%method",
          "formula",
          "compound",
          "isotopologue",
          "isotopologue%type"
          ],
      "id": "Metabolites",
      "sort_by": [
        "assignment"
      ],
      "sort_order": "ascending",
      "table": "measurement",
      "value_type": "matrix",
      "values_to_str": "True"
    },
    "Units": {
      "required": "True",
      "fields": [
        "intensity%type"
      ],
      "id": "Units",
      "table": "measurement",
      "value_type": "str"
    }
  },
  "PROJECT": {
    "ADDRESS": {
      "required": "True",
      "fields": [
        "address"
      ],
      "id": "ADDRESS",
      "table": "project",
      "value_type": "str"
    },
    "DEPARTMENT": {
      "required": "True",
      "fields": [
        "department"
      ],
      "id": "DEPARTMENT",
      "table": "project",
      "value_type": "str"
    },
    "EMAIL": {
      "required": "True",
      "fields": [
        "PI_email"
      ],
      "id": "EMAIL",
      "table": "project",
      "value_type": "str"
    },
    "FIRST_NAME": {
      "required": "True",
      "fields": [
        "PI_first_name"
      ],
      "id": "FIRST_NAME",
      "table": "project",
      "value_type": "str"
    },
    "INSTITUTE": {
      "required": "True",
      "fields": [
        "institution"
      ],
      "id": "INSTITUTE",
      "table": "project",
      "value_type": "str"
    },
    "LAST_NAME": {
      "required": "True",
      "fields": [
        "PI_last_name"
      ],
      "id": "LAST_NAME",
      "table": "project",
      "value_type": "str"
    },
    "PHONE": {
      "required": "True",
      "fields": [
        "phone"
      ],
      "id": "PHONE",
      "table": "project",
      "value_type": "str"
    },
    "PROJECT_SUMMARY": {
      "required": "True",
      "fields": [
        "description"
      ],
      "id": "PROJECT_SUMMARY",
      "table": "project",
      "value_type": "str"
    },
    "PROJECT_TITLE": {
      "required": "True",
      "fields": [
        "title"
      ],
      "id": "PROJECT_TITLE",
      "table": "project",
      "value_type": "str"
    }
  },
  "SAMPLEPREP": {
    "SAMPLEPREP_PROTOCOL_FILENAME": {
      "delimiter": ";",
      "fields": [
        "filename"
      ],
      "for_each": "True",
      "id": "SAMPLEPREP_PROTOCOL_FILENAME",
      "required": "False",
      "sort_by": [
        "order"
      ],
      "sort_order": "ascending",
      "table": "protocol",
      "test": "type=sample_prep",
      "value_type": "str"
    },
    "SAMPLEPREP_PROTOCOL_ID": {
      "delimiter": ";",
      "fields": [
        "id"
      ],
      "for_each": "True",
      "id": "SAMPLEPREP_PROTOCOL_ID",
      "required": "True",
      "sort_by": [
        "order"
      ],
      "sort_order": "ascending",
      "table": "protocol",
      "test": "type=sample_prep",
      "value_type": "str"
    },
    "SAMPLEPREP_SUMMARY": {
      "delimiter": ";",
      "fields": [
        "description"
      ],
      "for_each": "True",
      "id": "SAMPLEPREP_SUMMARY",
      "required": "True",
      "sort_by": [
        "order"
      ],
      "sort_order": "ascending",
      "table": "protocol",
      "test": "type=sample_prep",
      "value_type": "str"
    }
  },
  "STUDY": {
    "ADDRESS": {
      "required": "True",
      "fields": [
        "address"
      ],
      "id": "ADDRESS",
      "table": "study",
      "value_type": "str"
    },
    "DEPARTMENT": {
      "required": "True",
      "fields": [
        "department"
      ],
      "id": "DEPARTMENT",
      "table": "study",
      "value_type": "str"
    },
    "EMAIL": {
      "required": "True",
      "fields": [
        "PI_email"
      ],
      "id": "EMAIL",
      "table": "study",
      "value_type": "str"
    },
    "FIRST_NAME": {
      "required": "True",
      "fields": [
        "PI_first_name"
      ],
      "id": "FIRST_NAME",
      "table": "study",
      "value_type": "str"
    },
    "INSTITUTE": {
      "required": "True",
      "fields": [
        "institution"
      ],
      "id": "INSTITUTE",
      "table": "study",
      "value_type": "str"
    },
    "LAST_NAME": {
      "required": "True",
      "fields": [
        "PI_last_name"
      ],
      "id": "LAST_NAME",
      "table": "study",
      "value_type": "str"
    },
    "PHONE": {
      "required": "True",
      "fields": [
        "phone"
      ],
      "id": "PHONE",
      "table": "study",
      "value_type": "str"
    },
    "STUDY_SUMMARY": {
      "required": "True",
      "fields": [
        "description"
      ],
      "id": "STUDY_SUMMARY",
      "table": "study",
      "value_type": "str"
    },
    "STUDY_TITLE": {
      "required": "True",
      "fields": [
        "title"
      ],
      "id": "STUDY_TITLE",
      "table": "study",
      "value_type": "str"
    }
  },
  "SUBJECT": {
    "SUBJECT_SPECIES": {
      "required": "True",
      "fields": [
        "species"
      ],
      "id": "SUBJECT_SPECIES",
      "table": "entity",
      "test": "type=subject",
      "value_type": "str"
    },
    "SUBJECT_TYPE": {
      "required": "True",
      "fields": [
        "species_type"
      ],
      "id": "SUBJECT_TYPE",
      "table": "entity",
      "test": "type=subject",
      "value_type": "str"
    },
    "TAXONOMY_ID": {
      "required": "True",
      "fields": [
        "taxonomy_id"
      ],
      "id": "TAXONOMY_ID",
      "table": "entity",
      "test": "type=subject",
      "value_type": "str"
    }
  },
  "SUBJECT_SAMPLE_FACTORS": {
    "no_id_needed": {
      "code": "mwtab_functions.create_subject_sample_factors(input_json)",
      "id": "no_id_needed",
      "value_type": "section",
      "required": "True",
    }
  },
  "TREATMENT": {
    "TREATMENT_PROTOCOL_FILENAME": {
      "delimiter": ";",
      "fields": [
        "filename"
      ],
      "for_each": "True",
      "id": "TREATMENT_PROTOCOL_FILENAME",
      "required": "False",
      "sort_by": [
        "id"
      ],
      "sort_order": "ascending",
      "table": "protocol",
      "test": "type=treatment",
      "value_type": "str"
    },
    "TREATMENT_PROTOCOL_ID": {
      "delimiter": ";",
      "fields": [
        "id"
      ],
      "for_each": "True",
      "id": "TREATMENT_PROTOCOL_ID",
      "required": "True",
      "sort_by": [
        "id"
      ],
      "sort_order": "ascending",
      "table": "protocol",
      "test": "type=treatment",
      "value_type": "str"
    },
    "TREATMENT_SUMMARY": {
      "delimiter": ";",
      "fields": [
        "description"
      ],
      "for_each": "True",
      "id": "TREATMENT_SUMMARY",
      "required": "True",
      "sort_by": [
        "id"
      ],
      "sort_order": "ascending",
      "table": "protocol",
      "test": "type=treatment",
      "value_type": "str"
    }
  }
}



nmr_directives = copy.deepcopy(ms_directives)
del nmr_directives["CHROMATOGRAPHY"]
del nmr_directives["MS"]
del nmr_directives["MS_METABOLITE_DATA"]

nmr_directives["ANALYSIS"]["ANALYSIS_TYPE"]["override"] = "NM"
nmr_directives["NM"] = {
    "ACQUISITION_TIME": {
      "fields": [
        "acquisition_time",
        "\" \"",
        "acquisition_time%units"
      ],
      "id": "ACQUISITION_TIME",
      "required": "False",
      "table": "protocol",
      "test": "machine_type=NMR",
      "value_type": "str"
    },
    "BASELINE_CORRECTION_METHOD": {
      "fields": [
        "baseline_correction_method"
      ],
      "id": "BASELINE_CORRECTION_METHOD",
      "required": "False",
      "table": "protocol",
      "test": "machine_type=NMR",
      "value_type": "str"
    },
    "CHEMICAL_SHIFT_REF_CPD": {
      "fields": [
        "chemical_shift_ref_cpd"
      ],
      "id": "CHEMICAL_SHIFT_REF_CPD",
      "required": "False",
      "table": "protocol",
      "test": "machine_type=NMR",
      "value_type": "str"
    },
    "INSTRUMENT_NAME": {
      "fields": [
        "instrument"
      ],
      "id": "INSTRUMENT_NAME",
      "required": "True",
      "table": "protocol",
      "test": "machine_type=NMR",
      "value_type": "str"
    },
    "INSTRUMENT_TYPE": {
      "fields": [
        "instrument_type"
      ],
      "id": "INSTRUMENT_TYPE",
      "required": "True",
      "table": "protocol",
      "test": "machine_type=NMR",
      "value_type": "str"
    },
    "NMR_EXPERIMENT_TYPE": {
      "fields": [
        "NMR_experiment_type"
      ],
      "id": "NMR_EXPERIMENT_TYPE",
      "required": "True",
      "table": "protocol",
      "test": "machine_type=NMR",
      "value_type": "str"
    },
    "NMR_PROBE": {
      "fields": [
        "NMR_probe"
      ],
      "id": "NMR_PROBE",
      "required": "False",
      "table": "protocol",
      "test": "machine_type=NMR",
      "value_type": "str"
    },
    "NMR_SOLVENT": {
      "fields": [
        "NMR_solvent"
      ],
      "id": "NMR_SOLVENT",
      "required": "False",
      "table": "protocol",
      "test": "machine_type=NMR",
      "value_type": "str"
    },
    "NMR_TUBE_SIZE": {
      "fields": [
        "NMR_tube_size",
        "\" \"",
        "NMR_tube_size%units"
      ],
      "id": "NMR_TUBE_SIZE",
      "required": "False",
      "table": "protocol",
      "test": "machine_type=NMR",
      "value_type": "str"
    },
    "PULSE_SEQUENCE": {
      "fields": [
        "pulse_sequence"
      ],
      "id": "PULSE_SEQUENCE",
      "required": "False",
      "table": "protocol",
      "test": "machine_type=NMR",
      "value_type": "str"
    },
    "RELAXATION_DELAY": {
      "fields": [
        "relaxation_delay",
        "\" \"",
        "relaxation_delay%units"
      ],
      "id": "RELAXATION_DELAY",
      "required": "False",
      "table": "protocol",
      "test": "machine_type=NMR",
      "value_type": "str"
    },
    "SHIMMING_METHOD": {
      "fields": [
        "shimming_method"
      ],
      "id": "SHIMMING_METHOD",
      "required": "False",
      "table": "protocol",
      "test": "machine_type=NMR",
      "value_type": "str"
    },
    "SPECTROMETER_FREQUENCY": {
      "fields": [
        "spectrometer_frequency",
        "\" \"",
        "spectrometer_frequency%units"
      ],
      "id": "SPECTROMETER_FREQUENCY",
      "required": "True",
      "table": "protocol",
      "test": "machine_type=NMR",
      "value_type": "str"
    },
    "STANDARD_CONCENTRATION": {
      "fields": [
        "standard_concentration",
        "\" \"",
        "standard_concentration%units"
      ],
      "id": "STANDARD_CONCENTRATION",
      "required": "False",
      "table": "protocol",
      "test": "machine_type=NMR",
      "value_type": "str"
    },
    "TEMPERATURE": {
      "fields": [
        "temperature",
        "\" \"",
        "temperature%units"
      ],
      "id": "TEMPERATURE",
      "required": "False",
      "table": "protocol",
      "test": "machine_type=NMR",
      "value_type": "str"
    },
    "WATER_SUPPRESSION": {
      "fields": [
        "water_suppression"
      ],
      "id": "WATER_SUPPRESSION",
      "required": "False",
      "table": "protocol",
      "test": "machine_type=NMR",
      "value_type": "str"
    }
  }

nmr_directives["NMR_METABOLITE_DATA"] = {
    "Data": {
      "required": "True",
      "collate": "assignment",
      "headers": [
        "\"Metabolite\"=assignment",
        "entity.id=intensity"
      ],
      "id": "Data",
      "sort_by": [
        "assignment"
      ],
      "sort_order": "ascending",
      "table": "measurement",
      "value_type": "matrix",
      "values_to_str": "True"
    },
    "Extended": {
      "required": "False",
      "exclusion_headers": [
        "id",
        "intensity",
        "intensity%type",
        "intensity%units",
        "assignment",
        "assignment%method",
        "entity.id",
        "protocol.id",
        "base_inchi",
        "representative_inchi",
        "isotopic_inchi",
        "peak_description",
        "peak_pattern",
        "proton_count",
        "transient_peak",
        "transient_peak%type",
        "compound"
      ],
      "fields_to_headers": "True",
      "headers": [
        "\"Metabolite\"=assignment",
        "\"sample_id\"=entity.id"
      ],
      "id": "Extended",
      "sort_by": [
        "assignment"
      ],
      "sort_order": "ascending",
      "table": "measurement",
      "value_type": "matrix",
      "values_to_str": "True"
    },
    "Metabolites": {
      "required": "True",
      "collate": "assignment",
      "headers": [
        "\"Metabolite\"=assignment"
      ],
      "optional_headers": [
          "assignment%method",
          "base_inchi",
          "representative_inchi",
          "isotopic_inchi",
          "peak_description",
          "peak_pattern",
          "proton_count",
          "transient_peak",
          "transient_peak%type",
          "compound"
          ],
      "id": "Metabolites",
      "sort_by": [
        "assignment"
      ],
      "sort_order": "ascending",
      "table": "measurement",
      "value_type": "matrix",
      "values_to_str": "True"
    },
    "Units": {
      "required": "True",
      "fields": [
        "intensity%type"
      ],
      "id": "Units",
      "table": "measurement",
      "value_type": "str"
    }
  }

nmr_binned_directives = copy.deepcopy(nmr_directives)
del nmr_binned_directives["NMR_METABOLITE_DATA"]
nmr_binned_directives["NMR_BINNED_DATA"] = {
    "Data": {
      "required": "True",
      "collate": "assignment",
      "headers": [
        "\"Bin range(ppm)\"=assignment",
        "entity.id=intensity"
      ],
      "id": "Data",
      "sort_by": [
        "assignment"
      ],
      "sort_order": "ascending",
      "table": "measurement",
      "value_type": "matrix",
      "values_to_str": "True"
    },
    "Units": {
      "required": "True",
      "fields": [
        "intensity%type"
      ],
      "id": "Units",
      "table": "measurement",
      "value_type": "str"
    }
  }
